@@ -33,7 +33,7 @@
 #include <range/v3/iterator/traits.hpp>
 #include <range/v3/utility/polymorphic_cast.hpp>
 
-#include <range/v3/detail/disable_warnings.hpp>
+#include <range/v3/detail/prologue.hpp>
 
 namespace ranges
 {
@@ -41,11 +41,8 @@ namespace ranges
     namespace detail
     {
         template<typename T>
-        std::pair<T *, std::ptrdiff_t> get_temporary_buffer_impl(
-            std::size_t count) noexcept
+        std::pair<T *, std::ptrdiff_t> get_temporary_buffer_impl(std::size_t n) noexcept
         {
-            RANGES_EXPECT(count >= 0);
-            std::size_t n = static_cast<std::size_t>(count);
             if(n > PTRDIFF_MAX / sizeof(T))
                 n = PTRDIFF_MAX / sizeof(T);
 
@@ -91,9 +88,9 @@ namespace ranges
             }
         };
 
-        template<typename T, typename... Args>
-        auto make_unique(Args &&... args) -> CPP_ret(std::unique_ptr<T>)( //
-            requires(!std::is_array<T>::value))
+        template(typename T, typename... Args)(
+            requires (!std::is_array<T>::value)) //
+        std::unique_ptr<T> make_unique(Args &&... args)
         {
             return std::unique_ptr<T>{new T(static_cast<Args &&>(args)...)};
         }
@@ -121,15 +118,17 @@ namespace ranges
             return *this;
         }
         CPP_member
-        auto operator=(Val const & val) -> CPP_ret(raw_storage_iterator &)( //
-            requires copy_constructible<Val>)
+        auto operator=(Val const & val) //
+            -> CPP_ret(raw_storage_iterator &)(
+                requires copy_constructible<Val>)
         {
             ::new((void *)std::addressof(*out_)) Val(val);
             return *this;
         }
         CPP_member
-        auto operator=(Val && val) -> CPP_ret(raw_storage_iterator &)( //
-            requires move_constructible<Val>)
+        auto operator=(Val && val) //
+            -> CPP_ret(raw_storage_iterator &)(
+                requires move_constructible<Val>)
         {
             ::new((void *)std::addressof(*out_)) Val(std::move(val));
             return *this;
@@ -140,14 +139,16 @@ namespace ranges
             return *this;
         }
         CPP_member
-        auto operator++(int) -> CPP_ret(void)( //
-            requires(!forward_iterator<O>))
+        auto operator++(int) //
+            -> CPP_ret(void)(
+                requires (!forward_iterator<O>))
         {
             ++out_;
         }
         CPP_member
-        auto operator++(int) -> CPP_ret(raw_storage_iterator)( //
-            requires forward_iterator<O>)
+        auto operator++(int) //
+            -> CPP_ret(raw_storage_iterator)(
+                requires forward_iterator<O>)
         {
             auto tmp = *this;
             ++out_;
@@ -205,16 +206,16 @@ namespace ranges
         }
     };
 
-    template<typename I>
-    auto iter_ref(I & i) -> CPP_ret(iterator_wrapper<I>)( //
+    template(typename I)(
         requires input_or_output_iterator<I>)
+    iterator_wrapper<I> iter_ref(I & i)
     {
         return i;
     }
 
     template<typename I>
-    struct readable_traits<iterator_wrapper<I>>
-      : meta::if_c<(bool)input_iterator<I>, readable_traits<I>, meta::nil_>
+    struct indirectly_readable_traits<iterator_wrapper<I>>
+      : meta::if_c<(bool)input_iterator<I>, indirectly_readable_traits<I>, meta::nil_>
     {};
 
     template<typename Val>
@@ -250,6 +251,6 @@ namespace ranges
     /// @}
 } // namespace ranges
 
-#include <range/v3/detail/reenable_warnings.hpp>
+#include <range/v3/detail/epilogue.hpp>
 
 #endif
